# -*- coding: utf-8 -*-
"""Terms & Conditions Widget Interface."""

# zope imports
from z3c.form.interfaces import ISingleCheckBoxWidget


class ITCWidget(ISingleCheckBoxWidget):
    """Custom Terms & Conditions widget."""
